;;==============================================================================================================
;; Include separate files
;;==============================================================================================================

__includes["player_class.nls" "agent_class.nls" "waterpath_class.nls" "water_class.nls" "drop_class.nls" "field_class.nls" "activity_class.nls" "tank_class.nls" "pump_class.nls" "outlet_class.nls" "source_class.nls" "legend_class.nls" ]

;;==============================================================================================================
;; For querying mysql
;;==============================================================================================================
extensions [sql]



globals 
[
  ;;==============================================================================================================
  ;; current-player : int
  ;;==============================================================================================================
  ;; The number of the current player. All actions concerning a player are made to him.
  ;;
  current-player
  
  ;;==============================================================================================================
  ;; legend-x : float
  ;;==============================================================================================================
  ;; xcor max for the legend.
  ;;
  legend-x
  
  ;;==============================================================================================================
  ;; legend-min-y : float
  ;;==============================================================================================================
  ;; ycor min for the legend.
  ;;
  legend-y
  
  
  ;;==============================================================================================================
  ;; activity-items : list of activities
  ;;==============================================================================================================
  ;; A holder for legend's item activityblock.
  ;;
  activity-items
  
  ;;==============================================================================================================
  ;; color-item : global color to make distinction beetween turtles
  ;;==============================================================================================================
  ;; A global variable to change color
  ;;
  color-item
  
  
  ;;==============================================================================================================
  ;; x-step : float
  ;;==============================================================================================================
  ;; step for x increase during the waterpath building.
  ;;
  x-step
  
  ;;==============================================================================================================
  ;; projects : list of projects
  ;;==============================================================================================================
  ;; The list of all available projects.
  ;;
  projects
  
  ;;==============================================================================================================
  ;; current-project : project
  ;;==============================================================================================================
  ;; The project chosen.
  ;;
  current-project
  
  db-activities
  
  ;;==============================================================================================================
  ;; flowing-resources : List of resources that flow in the watershed
  ;; fixed-resources : List of resources that are motionless
  ;;==============================================================================================================
  ;; All the ressources used in this project.
  
  flowing-resources
  
  fixed-resources
  
  ;;==============================================================================================================
  ;;player-state? boolean : helpful to change players' type without the setup button 
  ;;==============================================================================================================
  ;; last value of agent? switch
  ;;
  player-state?
  
  ;;==============================================================================================================
  ;;next? : define the next time step 
  ;;==============================================================================================================
  next?
  
  ;;==============================================================================================================
  ;;wagbank? : a boolean to tell when to pay the players 
  ;;==============================================================================================================
  wagbank?
  
  ;;==============================================================================================================
  ;;some globals for experiments
  ;;---------------------------
  ;;==============================================================================================================
  
  ;;==============================================================================================================
  ;;group: a list that will contain the traitment number i and the index of group j G(i,j) 
  ;;==============================================================================================================
  group
]

;;==============================================================================================================
;; create-drop. (observer)
;; Create a drop.
;;==============================================================================================================
to-report create-drop [ knd ld clr ]
  let my-drop []
 
  hatch-drops ld 
  [
    set hidden? true
    set load 1
    set kind knd
    set color clr
    set my-drop lput self my-drop
  ]
  
  report my-drop
end

;;==============================================================================================================
;; get-total-load. (observer)
;; Compute the total load of this water.
;;==============================================================================================================
to-report get-total-load [ wtr ]
  
  report length [list-of-drops] of wtr

end
;;==============================================================================================================
;; get-water-for. (observer)
;; Get drops from water to create new one.
;;==============================================================================================================
to-report create-water-with [ wtr amnt ]
  let new-water nobody
  hatch-waters 1 
  [ 
    setup-water
    set new-water self
  ]
  let new-list [] 
 
  ; shuffle the drops in the list to take n of them randomly
   ask wtr
   [
     set list-of-drops shuffle list-of-drops
   ]
   
   ifelse amnt <= get-total-load wtr
   [set new-list n-of amnt [list-of-drops] of wtr]
   [set new-list [list-of-drops] of wtr]
   
   
  ;; update the list of the new water.
  ask new-water [ set list-of-drops new-list ]
  
  
  ask wtr [ set list-of-drops filter [not member? ? new-list] list-of-drops ]
  
  report new-water
 
end
;;==============================================================================================================
;; mix-waters. (observer)
;; Mix two waters into one.
;;==============================================================================================================
to mix-waters [ wtr1 wtr2 ]
  ;; Mix drops in wtr1 with ones in wtr2
  ask wtr1 [
    set list-of-drops sentence list-of-drops [list-of-drops] of wtr2
  ]
  ;; Kill useless water
  ask wtr2 [ die ]
end

to get-additional-resources-for-activity [ fld ]

  let new-consumed[]
  let p-rsc nobody
  let fieldowner players with [user-id = user-id-from-number [owner] of fld]
    
  ask fieldowner
  [ set p-rsc player-resources ]
  
  let act one-of activities in-radius 1 

  
  ask act [
    foreach fixed-resources-consumed [
      
      let knd-of-rsc first ?
      let match-rsc filter [ item 0 ? = knd-of-rsc ] p-rsc
      
      
      ;; If there is a match resource.
      if not empty? match-rsc and item 1 first match-rsc > 0 and (last ? - item 1 ?) > 0
      [
        ;; Enough resource of that kind.
        if item 1 first match-rsc >= (last ? - item 1 ?)
        [ 
         let dif last ? - item 1 ?
         
         ask fieldowner
         [
           player-set-resource knd-of-rsc ((player-get-resource knd-of-rsc) - dif) 
         ] 
          
          ;;set load = needed
          set ? replace-item 1 ? last ?
        ]
        
      ]
      set new-consumed lput ? new-consumed
      
    ]
    ; set the new list of resources consumed
   
      set fixed-resources-consumed new-consumed
  ]
end


;;==============================================================================================================
;; feed-field-with-water. (observer)
;; Feed the field.
;;==============================================================================================================
to feed-field-with-water [ wtr fld ]
 
  let new-consumed[]
  let act one-of activities in-radius 1 
  
  ;get the total consumed of the activity
  let ttl-cnsmed get-total-consume act
  
  
  let water-cnsmed create-water-with wtr ttl-cnsmed
  
  
  ask act [
    foreach flowing-resources-consumed [
      let knd first ?
      let nbr length filter [[kind] of ? = knd] [list-of-drops] of water-cnsmed
      
      set ? replace-item 1 ? (item 1 ? + nbr)
       
      set new-consumed lput ? new-consumed
      ]
            
    ; set the new list of resources consumed
      set flowing-resources-consumed new-consumed    
  ]
  
  ; kill the consumed water
  ask water-cnsmed [die]
  
  ask act [activity-update-view]
  
end

;;==============================================================================================================
;; Get the user-id from the number. (observer)
;; Report the user-id.
;;==============================================================================================================
to-report user-id-from-number [ num ]
  report [user-id] of one-of players with [ number = num ]
end



;;==============================================================================================================
;; Startup the server and wait for incoming computer client.(observer)
;;==============================================================================================================
to startup
  hubnet-set-client-interface "COMPUTER" []
  hubnet-reset
end


;;==============================================================================================================
;; update-list-ressources. (oberserver)
;; Update the list of resources to match the current project.
;;==============================================================================================================
to update-list-ressources
  ;; Get the list of available ressources
  sql:exec-query word "SELECT libelle FROM kit_has_ressources k inner join `ressources` r on k.ressource_id=r.id WHERE kit_id = " (item 2 current-project)[] 
  let ressources sql:fetch-resultset

  ;; Iterate through the result of the request
  set flowing-resources []
  set fixed-resources []
  
  foreach ressources
  [
    ifelse item 0 ? != "WAG"
    [
      ifelse item 0 ? = "eau propre"
         [set flowing-resources lput (list (item 0 ?) 84) flowing-resources]
         [
           ifelse item 0 ? = "eau sale" or item 0 ? = "eau verte"
           [
             set flowing-resources lput (list(item 0 ?) red) flowing-resources
           ]
           [
             set flowing-resources lput (list(item 0 ?) green) flowing-resources
           ]
         ]
         
    ]
    [
      set fixed-resources lput (list (item 0 ?) yellow ) fixed-resources
    ]
  ]
end


;;==============================================================================================================
;; Setup all the core things like database and legend's items.(observer)
;;==============================================================================================================
to setup
  reset-ticks
  ;; Configure SQL
  ifelse local? [
    sql:configure "defaultconnection" [["host" "localhost"] ["port" 3306] ["user" "root"] ["password" ""] ["database" "interwag"]]
  ]
  [
    sql:configure "defaultconnection" [["host" "193.49.41.13"] ["port" 3306] ["user" "interwager"] ["password" "bliblablop"] ["database" "interwag"]]
  ]
  
  ;; Get the list of available projects
  sql:exec-query "SELECT * FROM projets" []
  set projects sql:fetch-resultset
  ;; Select the first project by default
  if not empty? projects [ set current-project (item 0 projects) ]

  update-list-ressources

  init-model
  
  set player-state? agent?
end

to init-model
  clear-patches
  clear-drawing
  clear-output
  
  set next? false
  set wagbank? false
  
  ask turtles with [ not is-player? self ] 
  [
    die
  ]
  
  ;; Reset the player's variables
  ask players [ die ]
  set current-player 0
  
  ;; Setup Legend
  set legend-x (0.1 * max-pxcor)
  set legend-y max-pycor
  

  set activity-items []
  set color-item 15
  
  ;; Set increment for Waterpath building.
  set x-step 0.5
  
  ;; Create the players
  create-players nb-players
  
  ;; init the players
  ask players
  [
    player-setup 
  ]
  
  ;; init the group of players.
  set group[]
  set group lput 1 group
  set group lput 1 group
   
   
  ;; Show the legend
  build-legend
end


;to create-turtle-from-legend-item [a-legend-item]
;Use the data loaded at project initialisation to create a turtle corresponding to the legend selected item.
;It requires to have a global variable storing the result of the sql query (variable activities in extract-activities-from-db).

to extract-activities-from-db
;; Extract the activities from the DB.
    sql:exec-query word "SELECT activite_id, titre, ressource_id, libelle, consomme, produit FROM kit_has_activite_has_ressources k1 INNER JOIN kit_has_activites k2 ON k2.id = k1.kit_has_activite_id INNER JOIN activites a1 ON a1.id = activite_id INNER JOIN ressources r1 ON r1.id = ressource_id WHERE k2.kit_id =" (item 2 current-project) []
    set db-activities sql:fetch-resultset
end

;;==============================================================================================================
;; Get the slope of a linear function given two points of it.
;;
;; sx: xcor of the path's start
;; sy: ycor of the path's start
;; ex: xcor of the path's end
;; ey: ycor of the path's end
;;==============================================================================================================
to-report get-slope [ sx sy ex ey ] ;; observer
  ifelse (ex - sx) != 0 [
    ;; Compute the slope
    let a (ey - sy) / (ex - sx)
    
    report a
  ]
  [ report 0 ]
end



;;==============================================================================================================
;; Get the y-intercept of a linear function given two points of it.
;;
;; sx: xcor of the path's start
;; sy: ycor of the path's start
;; ex: xcor of the path's end
;; ey: ycor of the path's end
;;==============================================================================================================
to-report get-y-intercept [ sx sy a ] ;; observer
                                      ;; Compute the y-intercept
  report sy - a * sx
end

;;==============================================================================================================
;; Handle the user interaction with the mouse over the world.
;;==============================================================================================================
to handle-mouse-down ;; observer
  if mouse-down?
  [
    ;; Get the nearest turtlesshow 
    let candidate min-one-of turtles with [ not (is-waterarea? self) and not is-drop? self] [distancexy mouse-xcor mouse-ycor]
    
    ;; Consider only the turtle really near
    if [distancexy mouse-xcor mouse-ycor] of candidate < 1
    [
      ;; Which legend item are we dragging ?
      ifelse  is-legend-item? candidate [ handle-legend-item candidate ]
      [
          ifelse is-tank? candidate [ handle-tank candidate ]  
          [
            ifelse is-source? candidate [ handle-source candidate ]  
            [
            ]
          ]
      ]
     ]
   ]
end

;;==============================================================================================================
;; Handle the user interation with the legend-item
;;==============================================================================================================
to handle-legend-item [ candidate ]
   
   ;; The WATCH primitive puts a "halo" around the watched turtle.
   watch candidate
   
   let loc-breed [item-breed] of candidate
     
     ask candidate
     [
       if loc-breed = "waterarea" [handle-waterpath-item ]
       if loc-breed = "source"    [handle-source-item ]
       if loc-breed = "tank"      [handle-tank-item]
       if loc-breed = "field"     [handle-field-item]
       if loc-breed = "activity"  [handle-activity-item]
     ]
    ;; reset the perspective on kill the legend item
    reset-perspective
    ask candidate
    [ ask out-link-neighbors [die ]
      die
    ]
     
  ;; When the user has released the item, we build the legend.
  build-legend
end

;;==============================================================================================================
;; set the breed variable the same as the legend-item
;;==============================================================================================================
to setup-breed
  set color [color] of myself
  set shape [shape] of myself
  set heading [heading] of myself
  set size [size] of myself   
  
  ;; hide the legend-item.
  ask myself[set hidden? true]
end

;;==============================================================================================================
;; Handle the user interaction with the clicked source. It allow the user to chose
;; the delivry of the source.
;;
;; candidate: the source clicked.
;;==============================================================================================================
to handle-source [ candidate ]
  ask candidate
  [
    source-handling
  ]
end

;;==============================================================================================================
;; Handle the user interaction with the clicked tank. 
;; candidate: the tank clicked.
;;==============================================================================================================
to handle-tank [ candidate ]
  ask candidate
  [
    tank-handling
  ]
end

;;==============================================================================================================
;; Model watershed
;;==============================================================================================================

to build-watershed
    every 0.01
  [
    ;; Process user interaction with the mouse over the world.
    handle-mouse-down
    display
  ]
end

;;==============================================================================================================
;; Save the current watershed in the database or a file
;;==============================================================================================================
to save-watershed
  ;;Procedure to save watershed in a file (or to save it in the interwag DB)
  let file-name user-new-file
  if file-name != false
  [ 
    export-world (word file-name".csv") 
  ]
  
end

;;==============================================================================================================
;; Load an existing watershed
;;==============================================================================================================
to load-watershed
  ;;Procedure to load watershed from a file (or to restore it from the interwag DB) 
  let file-name user-file
  if file-name != false
  [
    import-world file-name
    
    ;;initialize players
    ask players [ 
      ifelse agent?
      [set user-id number]
      [set user-id 0]
    ]
      
    ;; reset ticks and ask go one time to apply changes
    reset-ticks
    go
    
  ]
  
  
end
  

;;==============================================================================================================
;; GO
;;==============================================================================================================
to go
  ;player job
    if agent? != player-state?
    [
      set current-player 0
      
      ask players
      [
        player-init
      ]
      set player-state? agent?
    ]
    
   if not agent?
   [
     ask players with [user-id != 0]
     [
       hubnet-send-override user-id other players "label" [""] 
     ]
   ]
    ;; Do the jobs
    
     ;outlet job
    ask outlets
    [
      outlet-job
    ]
    
    ;;water job
    ask waters
    [ 
      water-job
    ]
       
    ; tank job
    ask tanks
    [
      tank-job
    ]
    
    ;pump job
    ask pumps
    [
      pump-job
    ]
        
    ; field job
    ask fields with [has-activity?]
    [
       field-job
    ]
        
    

    ;; process incoming messages and respond to them (if needed)
    ;; listening for messages outside of the every block means that messages
    ;; get processed and responded to as fast as possible
    if not agent?
    [
      listen-clients
    ]
    ;; display (and tick) causes world updates messages to be sent to clients
    ;; these types of messages should only be sent inside an every block.
    ;; otherwise, the messages will be created as fast as possible
    ;; and consume your bandwidth.
    display
    
    
    ;; show next time step only if we press in "start new round" button.
    if next?
    [
      tick
      set next? false 
      
         
    ]    
    
    
    ;; a way to pay player after the end of a time step ( looking for a better way)
    if not any? waters and wagbank? = true
    [
      if not agent?
      [
        ask players with [user-id != 0]
        [
          hubnet-reset-perspective user-id
        ]
      ]  
       ;; check and pay the player if his activity is productive
      ask fields with [has-activity?]
      [
        let act activities in-radius 1
        
        ask act[
          activity-check-and-pay myself
        ]
      ]
      
      ;; important to pay only one time.
      set wagbank? false
    
    ]
end

to saving-process
    
    let file-players         (word "G"first group last group"players.csv")
    let file-sources         (word "G"first group last group"sources.csv")
    let file-fields          (word "G"first group last group"fields.csv")
    
  ;; write the files header and the initial data. 
  if ticks = 0
  [   
    write-in-csv-file file-players       ["i" "j" "player" "WAG" "tick" "agent?"]
    write-in-csv-file file-sources       ["i" "j" "type" "quantity" "tick"]
    write-in-csv-file file-fields        ["i" "j" "field" "activity" "player" "tick"]
  ]
  
  ; to write in players'file
  ask players
  [
    let nline (list first group last group number player-get-resource "WAG" ticks agent? )
    
    write-in-csv-file file-players nline
  ]
  
  ; to write in sources' file
  ask sources
  [
    let nline (list first group last group "public/natural" delivery ticks )
    
    write-in-csv-file file-sources nline
  ]
  
  ; to write in fields' file
   ask fields
  [
    let nline (list first group last group who)
    
    let act one-of activities in-radius 1
    set nline lput [label] of act nline
   
    set nline lput owner nline
    set nline lput ticks nline
    
    
    write-in-csv-file file-fields nline
  ]
  
end

;;==============================================================================================================
;; Clean-world : acts like a garbage collector.
;;==============================================================================================================
to clean-world
  
  ; remove from world all inactive activities
  ask activities with [hidden?]
  [ 
    ask out-link-neighbors [die ]
    die
  ]
  
  
end

;;==============================================================================================================
;; Start a new round.
;;==============================================================================================================
to start-new-round
  
  ;; if a round isn't ongoing
  if not any? waters
  [
    
    ;; ask for the saving of results
    saving-process
    
    set next? true
    set wagbank? true
       
    
    ; restart the pumps
    ask pumps
    [
      pump-init
    ]
        
    ; reset the load of each activity
    ask activities
    [
      activity-init
    ]
    
    
    ; give the possibility to player who owns a field to change his activity before next round
    ask fields with [ has-activity?]
    [
      ;; get the id of the player who owns the field
      let uid (user-id-from-number owner)
      
      ;; is there a client associated with this uid ?
      if uid != 0
      [
        if not (agent?)
        [
          player-change-activity self uid
        ]
      ]
    ]
          
    ; ask tanks' managers to  set the next delivery.
    ask tanks
    [
      ;; get the id of the player who manages the tank
      let uid (user-id-from-number manager)
      
      ;; is there a client associated with this uid ?
      if uid != 0
      [
        if not (agent?)
        [player-define-tank-delivery self uid]
      ]
      
    ]
   
    ; ask players to invest in public infrastructure.
    ask players with [user-id != 0]
    [
      if not (agent?)
      [
        player-invest-pi
      ]
    ]
    
    
    ; clean the world : delete unused elements (old activities)
    clean-world
    
    ;; get the additional resources needed by the activity only if it's next round.
    ask fields with [has-activity?]
    [
      get-additional-resources-for-activity self      
    ]
        
           
    ;; in each round, we ask sources to release water
    go-sources
  ]

end

;;==============================================================================================================
;; Report if a couple of cor is inside the world.
;; x: xcor
;; y: ycor
;;==============================================================================================================
to-report is-inside-world? [ x y ]
  report (x >= 0 and x <= max-pxcor) and (y >= 0 and y <= max-pycor)
end



;;==============================================================================================================
;; Report if a couple of cor is inside the legend.
;; x: xcor
;; y: ycor
;;==============================================================================================================
to-report is-inside-legend? [ x y ]
  report (x >= 0 and x <= legend-x) and (y <= max-pycor)
end



;;==============================================================================================================
;; Make rain !
;;==============================================================================================================
to go-rain
  user-message (word "Reviser cette procedure")
  
  ;;ask waterpath 
  ;;[
  ;;  if (not any? waters in-radius 2) and (self != waterpath-item)
  ;;  [
  ;;    if random 2 = 1
  ;;    [
  ;;      hatch-waters 1
  ;;      [
  ;;        set color sky
  ;;        set shape "drop"
  ;;        set size 2
  ;;        
  ;;        ;; Randomize the rain
  ;;        let rand-xcor xcor + random-normal 0 1
  ;;        let rand-ycor ycor + random-normal 0 1
  ;;        if is-inside-world? rand-xcor rand-ycor
  ;;        [
  ;;          set xcor rand-xcor
  ;;          set ycor rand-ycor
  ;;        ]
  ;;        
  ;;        set crossed-waterpath []
  ;;      ]
  ;;    ]
  ;;  ]
  ;;]
end

;;==============================================================================================================
;; Activate the sources.
;;==============================================================================================================
to go-sources
  ask sources
  [
    source-go
  ]
  ask tanks
  [
    tank-go
  ]
end



;;==============================================================================================================
;; Listen for incoming client events.
;;==============================================================================================================
to listen-clients
  ;; as long as there are more messages from the clients
  ;; keep processing them.
  while [ hubnet-message-waiting? ]
  [
    ;; get the first message in the queue
    hubnet-fetch-message
    
    ;; New client ?
    ifelse hubnet-enter-message? 
    [ 
      ;; Can we accept this new client ?
      ifelse count players with [ user-id != 0 ] < nb-players
      [ associate-new-player ]
      [ user-message (word "The client " hubnet-message-source " have no player left for him.") ]
    ]
    ;; Command message ?
    [
      ifelse hubnet-exit-message? ;; when clients exit we get a special message
      [ remove-player ]
      [ ask players with [user-id = hubnet-message-source ]
        [ execute-command hubnet-message-tag ] ;; otherwise the message means that the user has
      ]                                        ;; done something in the interface hubnet-message-tag
                                               ;; is the name of the widget that was changed
    ]
  ]
end



;;==============================================================================================================
;; Associates a client/agent with a player.
;;==============================================================================================================
to associate-new-player
  ask min-one-of players with [ user-id = 0  ] [ number ]
  [
    player-client-association
  ] 
end



;;==============================================================================================================
;; Remove the association between a player and a client.
;;==============================================================================================================
to remove-player
  ask players with [user-id = hubnet-message-source]
  [ 
        player-remove
  ]
  
end



;;==============================================================================================================
;; Associates a client with a player.
;;==============================================================================================================
to execute-command [command]
  ;; The user is chosen his player number.
  if command = "<"
  [ 
    ifelse number > 1 
    [ set number (number - 1) ]
    [ set number 1 ]
  ]
  if command = ">"
  [
    ifelse number < nb-players
    [ set current-player (number + 1) ]
    [ set number nb-players ]
  ]
  
  if command = "View"
  [
;    let x round item 0 hubnet-message
;    let y round item 1 hubnet-message
;    show hubnet-message-source
;    show x
;    show y
    
;    if (not on-going?)
;    [
;      let candidate min-one-of turtles [distancexy x y]
;      show candidate
      
;   ]
  ]
  
    
end



;;==============================================================================================================
;; Whenever something in world changes that should be displayed in
;; a monitor on the client send the information back to the client
;;==============================================================================================================
to send-info-to-clients ;; turtle procedure
  hubnet-send user-id "location" (word "(" pxcor "," pycor ")")
end



;;==============================================================================================================
;; Update the UI to show the current kit.
;;==============================================================================================================
to-report project-chooser
  if not empty? current-project [ report (item 3 current-project) ]
end

;;==============================================================================================================
;; Select the previous kit in the list.
;;==============================================================================================================
to project-left
  if ((position current-project projects) - 1) >= 0 
  [
     set current-project (item ((position current-project projects) - 1) projects)
     update-list-ressources
     init-model
  ]
  
end

;;==============================================================================================================
;; Select the next kit in the list.
;;==============================================================================================================
to project-right
  if ((position current-project projects) + 1) < length projects 
  [
     set current-project (item ((position current-project projects) + 1) projects) 
     update-list-ressources
     init-model
  ]
end


;;==============================================================================================================
;; write-in-csv-file: Write the result of the game in a file.
;; filename: the name of the file.
;; items: list of the data (or headers) to write.
;;==============================================================================================================
to write-in-csv-file [ filename items ]

  if is-list? items and not empty? items
  [ 
    ;; the path of the file will be defined by the experimenter.
    
    file-open (word "../results/"filename)
    
    ;; quote non-numeric items
    set items map quote items
    ;; print the items
    ;; if only one item, print it.
    ifelse length items = 1 
    [ 
      file-print first items
    ]
    [
      file-print reduce [ (word ?1 "," ?2) ] items
    ]
    
    ;; close-up
    file-close
  ]
end

;;==============================================================================================================
;; quote : quote non-numeric items.
;;==============================================================================================================
to-report quote [ one-item ]
  
  ifelse is-number? one-item
  [ 
    report one-item 
  ]
  [ 
    report (word "\"" one-item "\"") 
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
297
16
1053
608
-1
-1
7.39
1
10
1
1
1
0
0
0
1
0
100
0
75
1
1
1
round
30.0

BUTTON
123
161
194
194
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
214
374
285
407
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
127
572
199
605
NIL
startup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
28
503
204
536
speed-of-drop
speed-of-drop
0.01
1
0.51
0.1
1
NIL
HORIZONTAL

BUTTON
28
459
91
492
Rain
go-rain
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
24
49
86
109
nb-players
4
1
0
Number

BUTTON
96
459
217
492
Start new round
start-new-round
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
52
281
126
316
<
project-left
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
128
282
203
316
>
project-right
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
29
237
230
282
Choose the project
project-chooser
0
1
11

TEXTBOX
13
568
163
606
END. Restart game \n(disconects payers)
12
0.0
1

TEXTBOX
6
213
172
238
3. Choose a modelling kit\n
12
0.0
1

TEXTBOX
8
127
256
172
2. Load the projects (local or dist database) \n    in ana-wag
12
0.0
1

BUTTON
30
361
155
394
Create/modify
build-watershed
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
8
338
203
368
4. Create your watershed
12
0.0
1

BUTTON
29
396
94
429
save
save-watershed
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
97
397
160
430
load
load-watershed
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
180
341
330
371
5. Launch listening of\n     players actions
12
0.0
1

TEXTBOX
10
439
191
469
6. Manage simulation events
12
0.0
1

SWITCH
27
160
117
193
local?
local?
1
1
-1000

SWITCH
164
66
267
99
agent?
agent?
1
1
-1000

TEXTBOX
10
25
210
55
1. Define the simulation context
12
0.0
1

INPUTBOX
89
49
156
109
initial-WAG
4
1
0
Number

@#$#@#$#@
## WHAT IS IT?

This template contains code that can serve as a starting point for creating new HubNet activities. It shares many of the basic procedures used by other HubNet activities, which are required to connect to and communicate with clients in Disease-like activities.

## HOW IT WORKS

In activities like Disease, each client controls a single turtle on the server.  These turtles are a breed called STUDENTS.  When a client logs in we create a new student turtle and set it up with the default attributes.  Students own a variable for every widget on the client that holds a state, that is, sliders, switches, choosers, and input boxes.  Whenever a user changes one of these elements on the client, a message is sent to the server.  The server catches the message and stores the result.  In this example a slider is used to demonstrate this behavior.  You can also send messages to the client-side widgets using hubnet-send.  Monitors on clients must be updated manually by the model, that is you must send a message to a monitor every time you want the value displayed to change. For example, if you have a monitor that displays the current location of the client's avatar, you must send a message to the client like this:

     hubnet-send "location" (word xcor " " ycor)

whenever the client moves.  Buttons on the client side send but do not receive messages.  When a user presses a button, a message is sent to the server.  The server catches the message and executes the appropriate commands.  In this case, the commands should always be turtle commands since the clients control only a single turtle.

## HOW TO USE IT

To start the activity press the GO button.  Ask students to login using the HubNet client or you can test the activity locally by pressing the LOCAL button in the HubNet Control Center. To see the view in the client interface check the Mirror 2D view on clients checkbox.  The clients can use the UP, DOWN, LEFT, and RIGHT buttons to move their avatar and change the amount they move each step by changing the STEP-SIZE slider.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

activity
false
0
Circle -7500403 true true 45 45 210
Rectangle -16777216 true false 60 60 240 255
Line -7500403 true 60 150 255 150

agent
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 75 60 105 90
Rectangle -16777216 true false 195 60 225 90
Rectangle -16777216 true false 105 120 195 135
Rectangle -16777216 true false 45 195 255 255
Rectangle -7500403 true true 150 210 240 225
Line -16777216 false 45 180 255 180

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

box 2
false
0
Polygon -7500403 true true 150 285 270 225 270 90 150 150
Polygon -13791810 true false 150 150 30 90 150 30 270 90
Polygon -13345367 true false 30 90 30 225 150 285 150 150

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

drop
false
0
Circle -7500403 true true 73 133 152
Polygon -7500403 true true 219 181 205 152 185 120 174 95 163 64 156 37 149 7 147 166
Polygon -7500403 true true 79 182 95 152 115 120 126 95 137 64 144 37 150 6 154 165

empty circle
false
0
Circle -7500403 false true 0 0 300

empty square
false
0
Rectangle -7500403 false true 30 30 270 270

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

field
false
0
Line -7500403 true 90 45 210 45
Line -7500403 true 90 45 30 90
Line -7500403 true 30 90 30 210
Line -7500403 true 30 210 90 270
Line -7500403 true 90 270 210 270
Line -7500403 true 210 270 270 210
Line -7500403 true 270 210 270 90
Line -7500403 true 270 90 210 45
Rectangle -7500403 true true 90 255 210 285

fire department
false
0
Polygon -7500403 true true 150 55 180 60 210 75 240 45 210 45 195 30 165 15 135 15 105 30 90 45 60 45 90 75 120 60
Polygon -7500403 true true 55 150 60 120 75 90 45 60 45 90 30 105 15 135 15 165 30 195 45 210 45 240 75 210 60 180
Polygon -7500403 true true 245 150 240 120 225 90 255 60 255 90 270 105 285 135 285 165 270 195 255 210 255 240 225 210 240 180
Polygon -7500403 true true 150 245 180 240 210 225 240 255 210 255 195 270 165 285 135 285 105 270 90 255 60 255 90 225 120 240
Circle -7500403 true true 60 60 180
Circle -16777216 false false 75 75 150

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

hexagonal prism
false
0
Rectangle -7500403 true true 90 90 210 270
Polygon -1 true false 210 270 255 240 255 60 210 90
Polygon -13345367 true false 90 90 45 60 45 240 90 270
Polygon -11221820 true false 45 60 90 30 210 30 255 60 210 90 90 90

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.1.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="test1" repetitions="1" runMetricsEveryStep="true">
    <go>go</go>
    <metric>count players</metric>
  </experiment>
</experiments>
@#$#@#$#@
VIEW
292
35
1050
663
0
0
0
1
1
1
1
1
0
1
1
1
0
100
0
75

MONITOR
70
21
157
70
location
NIL
0
1

INPUTBOX
6
115
56
175
drops
0
1
0
String

MONITOR
4
90
234
139
Message
NIL
0
1

MONITOR
6
175
77
224
Seconds left
NIL
0
1

TEXTBOX
59
140
221
174
Press the \"return\" key to submit.
14
0.0
1

TEXTBOX
10
287
239
441
NOTE:\nTo change activity:\n\n- Wait until the message \"You can change your activity\" appears in the Message monitor\n\n- Click on the activity you want in the legend in order to replace the one in your watched field.\n\n- If all goes well, the message \"You've changed your activity\" will appear.
11
0.0
1

@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
